# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016-2017 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/

__authors__ = ["H. Payno"]
__license__ = "MIT"
__date__ = "06/07/2019"


from silx.gui import qt
from silx.gui.plot import LegendSelector
from Orange.widgets import gui
from Orange.widgets.widget import OWWidget
from Orange.widgets.settings import Setting
from xas.core.types import XASObject, Spectrum
import xas.core.process.exafs
from ..progress import QProgress
from xas.gui.XasObjectViewer import XasObjectViewer, _CurveOperation, ViewType
from PyMca5.PyMcaGui.physics.xas.XASPostEdgeParameters import XASPostEdgeParameters
import logging

_logger = logging.getLogger(__file__)


def _exafs_signal_plot(spectrum):
    assert isinstance(spectrum, Spectrum)
    missing_keys = spectrum.get_missing_keys(('EXAFSKValues', 'EXAFSSignal'))
    if missing_keys:
        _logger.error('missing keys:', missing_keys, 'unable to compute exafs signal plot')
        return
    k = spectrum["EXAFSKValues"]
    if "KMin" not in spectrum:
        spectrum["KMin"] = k.min()
    if "KMax" not in spectrum:
        spectrum["KMax"] = k.max()

    idx = (spectrum["EXAFSKValues"] >= spectrum["KMin"]) & \
          (spectrum["EXAFSKValues"] <= spectrum["KMax"])
    x = spectrum["EXAFSKValues"][idx]
    y = spectrum["EXAFSSignal"][idx]
    return _CurveOperation(x=x, y=y, legend="EXAFSSignal")


def _exafs_postedge_plot(spectrum):
    assert isinstance(spectrum, Spectrum)
    missing_keys = spectrum.get_missing_keys(('EXAFSKValues', 'PostEdgeB'))
    if missing_keys:
        _logger.error('missing keys:', missing_keys, 'unable to compute exafs postedge plot')
        return
    k = spectrum["EXAFSKValues"]
    if "KMin" not in spectrum:
        spectrum["KMin"] = k.min()
    if "KMax" not in spectrum:
        spectrum["KMax"] = k.max()

    idx = (spectrum["EXAFSKValues"] >= spectrum["KMin"]) & \
          (spectrum["EXAFSKValues"] <= spectrum["KMax"])

    x = spectrum["EXAFSKValues"][idx]
    y = spectrum["PostEdgeB"][idx]
    return _CurveOperation(x=x, y=y, legend="PostEdge")


def _exafs_knots_plot(spectrum):
    assert isinstance(spectrum, Spectrum)
    missing_keys = spectrum.get_missing_keys(('KnotsX', 'KnotsY'))
    if missing_keys:
        _logger.error('missing keys:', missing_keys, 'unable to compute exafs knot plot')
        return
    x = spectrum["KnotsX"]
    y = spectrum["KnotsY"]
    return _CurveOperation(x=x, y=y, legend="Knots", linestyle="", symbol="o")


class ExafsWindow(qt.QMainWindow):
    def __init__(self, parent=None):
        qt.QMainWindow.__init__(self, parent)
        mapKeys = ['Mu', 'EXAFS/EXAFSKValues', 'EXAFS/EXAFSSignal',
                   'EXAFS/PostEdgeB']
        self.xasObjViewer = XasObjectViewer(mapKeys=mapKeys)
        self.xasObjViewer._spectrumView._plot.getXAxis().setLabel("K")
        self.xasObjViewer._spectrumView._plot.getYAxis().setLabel("Normalized Units")
        self.setCentralWidget(self.xasObjViewer)

        # pymca window
        self._pymcaWindow = XASPostEdgeParameters(parent=self)
        dockWidget = qt.QDockWidget(parent=self)
        dockWidget.setWidget(self._pymcaWindow)
        self.addDockWidget(qt.Qt.RightDockWidgetArea, dockWidget)
        dockWidget.setAllowedAreas(qt.Qt.RightDockWidgetArea | qt.Qt.LeftDockWidgetArea)
        dockWidget.setFeatures(qt.QDockWidget.NoDockWidgetFeatures)
        self.setWindowFlags(qt.Qt.Widget)

        # legend selector
        self.legendDockWidget = LegendSelector.LegendsDockWidget(parent=self,
                                                                 plot=self.xasObjViewer._spectrumView._plot)
        self.legendDockWidget.setAllowedAreas(qt.Qt.RightDockWidgetArea | qt.Qt.LeftDockWidgetArea)
        self.legendDockWidget.setFeatures(qt.QDockWidget.NoDockWidgetFeatures)
        self.addDockWidget(qt.Qt.RightDockWidgetArea, self.legendDockWidget)

        # volume key selection
        self.addDockWidget(qt.Qt.RightDockWidgetArea,
                           self.xasObjViewer._mapView.keySelectionDocker)

        # plot settings
        for ope in (_exafs_signal_plot, _exafs_postedge_plot, _exafs_knots_plot,):
            self.xasObjViewer._spectrumView.addCurveOperation(ope)

        self.setWindowFlags(qt.Qt.Widget)

        # connect signal / slot
        self.xasObjViewer.viewTypeChanged.connect(self._updateLegendView)

        # set up
        self._updateLegendView()

    def getNCurves(self):
        return len(self.xasObjViewer.getAllCurves())

    def _updateLegendView(self):
        self.legendDockWidget.setVisible(self.xasObjViewer.getViewType() is ViewType.spectrum)
        self.xasObjViewer._mapView.keySelectionDocker.setVisible(self.xasObjViewer.getViewType() is ViewType.map)


class ExafsOW(OWWidget):
    """
    Widget used for signal extraction
    """
    name = "exafs"
    id = "orange.widgets.xas.exafs"
    description = "Progress signal extraction"
    icon = "icons/exafs.png"
    priority = 3
    category = "esrfWidgets"
    keywords = ["spectroscopy", "signal"]

    want_main_area = True
    resizing_enabled = True

    inputs = [("spectra", XASObject, "process")]
    outputs = [("spectra", XASObject), ("curves", tuple)]

    process_function = xas.core.process.exafs.PyMca_exafs

    _pymcaSettings = Setting(dict())
    """Store the configuration of the PyMca XASClass"""

    def __init__(self):
        super().__init__()
        self._window = ExafsWindow()
        layout = gui.vBox(self.mainArea, 'fourier transform').layout()
        layout.addWidget(self._window)
        self._latest_xas_obj = None

        self._window.xasObjViewer._spectrumView._plot.getXAxis().setLabel("K")
        self._window.xasObjViewer._spectrumView._plot.getYAxis().setLabel("Normalized Units")

        # progress
        self._progress = gui.ProgressBar(self, 100)

        # manage settings
        if self._pymcaSettings != dict():
            self._window._pymcaWindow.setParameters(self._pymcaSettings)

        # signal / slot connection
        _sig = self._window._pymcaWindow.sigPostEdgeParametersSignal
        _sig.connect(self._updateProcess)

    def _updateProcess(self):
        self._update_settings()
        if self._latest_xas_obj:
            self.process(self._latest_xas_obj)

    def process(self, xas_obj):
        if xas_obj is None:
            return

        self._latest_xas_obj = xas_obj.copy()
        process_obj = QPyMca_exafs()
        process_obj._advancement.sigProgress.connect(self._setProgressValue)
        process_obj.setProperties({'_pymcaSettings': self._window._pymcaWindow.getParameters()})
        xas_obj = process_obj.process(xas_obj)
        process_obj._advancement.sigProgress.disconnect(self._setProgressValue)
        self._window.xasObjViewer.setXASObj(xas_obj)

        # emit signal for the plot
        self.send("spectra", xas_obj)

    def _update_settings(self):
        self._pymcaSettings = self._window._pymcaWindow.getParameters()

    def _setProgressValue(self, value):
        self._progress.widget.progressBarSet(value)


class QPyMca_exafs(xas.core.process.exafs.PyMca_exafs):
    def __init__(self):
        xas.core.process.exafs.PyMca_exafs.__init__(self)
        self._advancement = QProgress('exafs')
