# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016-2017 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/

__authors__ = ["H. Payno"]
__license__ = "MIT"
__date__ = "06/07/2019"


from silx.gui.plot import LegendSelector
from Orange.widgets import gui
from Orange.widgets.widget import OWWidget
from Orange.widgets.settings import Setting
from silx.gui import qt
from xas.core.types import XASObject, Spectrum
import xas.core.process.k_weight
from ..progress import QProgress
from xas.gui.XasObjectViewer import XasObjectViewer, _CurveOperation, ViewType
import logging

_logger = logging.getLogger(__file__)


def _normalized_exafs(spectrum):
    assert isinstance(spectrum, Spectrum)
    missing_keys = spectrum.get_missing_keys(('EXAFSKValues', 'EXAFSNormalized'))
    if missing_keys:
        _logger.error('missing keys:', missing_keys,
                      'unable to compute normalized EXAFS')
        return None

    if spectrum["KWeight"]:
        if spectrum["KWeight"] == 1:
            ylabel = "EXAFS Signal * k"
        else:
            ylabel = "EXAFS Signal * k^%d" % spectrum["KWeight"]
    else:
        ylabel = "EXAFS Signal"

    idx = (spectrum["EXAFSKValues"] >= spectrum["KMin"]) & \
          (spectrum["EXAFSKValues"] <= spectrum["KMax"])

    return _CurveOperation(x=spectrum["EXAFSKValues"][idx],
                           y=spectrum["EXAFSNormalized"][idx],
                           legend="Normalized EXAFS",
                           ylabel=ylabel)


def _ft_window_plot(spectrum):
    assert isinstance(spectrum, Spectrum)
    missing_keys = spectrum.ft.get_missing_keys(('K', 'WindowWeight'))
    if missing_keys:
        _logger.error('missing keys:', missing_keys,
                      'unable to compute normalized EXAFS')
        return None

    return _CurveOperation(x=spectrum.ft['K'],
                           y=spectrum.ft["WindowWeight"],
                           legend="FT Window",
                           yaxis="right",
                           color="red")


class KWeightWindow(qt.QMainWindow):
    def __init__(self, parent=None):
        qt.QMainWindow.__init__(self, parent)
        # xas object viewer
        self.xasObjViewer = XasObjectViewer(mapKeys=['Mu'])
        self.xasObjViewer._spectrumView._plot.getXAxis().setLabel("K")
        self.setCentralWidget(self.xasObjViewer)

        # k wright widget
        self._k_widget = qt.QWidget(parent=self)
        self._k_widget.setLayout(qt.QHBoxLayout())
        self._k_widget.layout().addWidget(qt.QLabel('k weight'))
        self._k_spin_box = qt.QSpinBox(parent=self)
        self._k_spin_box.setRange(0, 3)
        self._k_widget.layout().addWidget(self._k_spin_box)
        dockWidget = qt.QDockWidget(parent=self)
        dockWidget.setWidget(self._k_widget)
        self.addDockWidget(qt.Qt.RightDockWidgetArea, dockWidget)
        dockWidget.setAllowedAreas(qt.Qt.RightDockWidgetArea | qt.Qt.LeftDockWidgetArea)
        dockWidget.setFeatures(qt.QDockWidget.NoDockWidgetFeatures)

        # legend selector
        self.legendDockWidget = LegendSelector.LegendsDockWidget(parent=self,
                                                                 plot=self.xasObjViewer._spectrumView._plot)
        self.legendDockWidget.setAllowedAreas(qt.Qt.RightDockWidgetArea | qt.Qt.LeftDockWidgetArea)
        self.legendDockWidget.setFeatures(qt.QDockWidget.NoDockWidgetFeatures)
        self.addDockWidget(qt.Qt.RightDockWidgetArea, self.legendDockWidget)

        # volume key selection
        self.addDockWidget(qt.Qt.RightDockWidgetArea,
                           self.xasObjViewer._mapView.keySelectionDocker)

        # plot settings
        for ope in (_normalized_exafs, _ft_window_plot):
            self.xasObjViewer._spectrumView.addCurveOperation(ope)

        self.setWindowFlags(qt.Qt.Widget)

        # connect signal / slot
        self.xasObjViewer.viewTypeChanged.connect(self._updateLegendView)

        # set up
        self._updateLegendView()

    def getNCurves(self):
        return len(self.plot.getAllCurves())

    def _updateLegendView(self, *args, **kwargs):
        self.legendDockWidget.setVisible(self.xasObjViewer.getViewType() is ViewType.spectrum)
        self.xasObjViewer._mapView.keySelectionDocker.setVisible(self.xasObjViewer.getViewType() is ViewType.map)


class KWeightOW(OWWidget):
    """
    Widget used for signal extraction
    """
    name = "k weight"
    id = "orange.widgets.xas.k_weight"
    description = "Progress k weight"
    icon = "icons/k_weight.png"
    priority = 2
    category = "esrfWidgets"
    keywords = ["spectroscopy", "signal", "k", "weight"]

    want_main_area = True
    resizing_enabled = True

    inputs = [("spectra", XASObject, "process")]
    outputs = [("spectra", XASObject), ("curves", tuple)]

    process_function = xas.core.process.k_weight.PyMca_k_weight

    _kWeightSetting = Setting(int(3))
    """Store the configuration of the PyMca XASClass"""

    def __init__(self):
        super().__init__()
        layout = gui.vBox(self.mainArea, 'fourier transform').layout()
        self._window = KWeightWindow(parent=self)
        layout.addWidget(self._window)
        self._latest_xas_obj = None

        # progress
        self._progress = gui.ProgressBar(self, 100)

        # manage settings
        if self._kWeightSetting != 3:
            self._window._k_spin_box.setValue(self._kWeightSetting)

        # signal / slot connection
        self._window._k_spin_box.valueChanged.connect(self._updateProcess)

    def _updateProcess(self, *arv, **kwargs):
        self._update_settings()
        if self._latest_xas_obj:
            self.process(self._latest_xas_obj)

    def process(self, xas_obj):
        if xas_obj is None:
            return

        self._latest_xas_obj = xas_obj.copy()
        process_obj = QPyMca_K_weight()
        process_obj._advancement.sigProgress.connect(self._setProgressValue)
        process_obj.setProperties({'_kWeightSetting': self._window._k_spin_box.value()})
        xas_obj = process_obj.process(xas_obj)
        process_obj._advancement.sigProgress.disconnect(self._setProgressValue)
        self._window.xasObjViewer.setXASObj(xas_obj)

        # emit signal for the plot
        self.send("spectra", xas_obj)

    def _update_settings(self):
        self._kWeightSetting = self._window._k_spin_box.value()

    def _setProgressValue(self, value):
        self._progress.widget.progressBarSet(value)


class QPyMca_K_weight(xas.core.process.k_weight.PyMca_k_weight):
    def __init__(self):
        xas.core.process.k_weight.PyMca_k_weight.__init__(self)
        self._advancement = QProgress('k weight')
