# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016-2017 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/

__authors__ = ["H. Payno"]
__license__ = "MIT"
__date__ = "06/07/2019"


from silx.gui import qt
from silx.gui.plot import LegendSelector
from Orange.widgets import gui
from Orange.widgets.widget import OWWidget
from Orange.widgets.settings import Setting
from PyMca5.PyMcaGui.physics.xas.XASNormalizationParameters import XASNormalizationParameters
from ..progress import QProgress
from xas.core.types import XASObject, Spectrum
from xas.gui.XasObjectViewer import XasObjectViewer, _CurveOperation, ViewType
import xas.core.process.normalization
import logging

_logger = logging.getLogger(__file__)


def _spectrum_plot(spectrum):
    assert isinstance(spectrum, Spectrum)
    missing_keys = spectrum.get_missing_keys(('Edge',))
    if missing_keys:
        _logger.error('missing keys:', missing_keys, 'unable to compute spectrum plot')
        return
    e0 = spectrum['Edge']
    x = spectrum.energy - e0
    y = spectrum.mu
    return _CurveOperation(x=x, y=y, legend='Spectrum', yaxis=None)


def _normalized_spectrum_plot(spectrum):
    assert isinstance(spectrum, Spectrum)
    missing_keys = spectrum.get_missing_keys(('Edge', 'NormalizedEnergy',
                                              'NormalizedPlotMin', 'NormalizedPlotMax',
                                              'NormalizedMu'))
    if missing_keys:
        _logger.error('missing keys:', missing_keys, 'unable to compute normalized spectrum plot')
        return None

    idx = (spectrum["NormalizedEnergy"] >= spectrum["NormalizedPlotMin"]) & \
          (spectrum["NormalizedEnergy"] <= spectrum["NormalizedPlotMax"])
    e0 = spectrum['Edge']
    x = spectrum["NormalizedEnergy"][idx] - e0
    y = spectrum["NormalizedMu"][idx]
    return _CurveOperation(x=x, y=y, legend='Normalized', yaxis='right')


def _post_plot(spectrum):
    assert isinstance(spectrum, Spectrum)
    missing_keys = spectrum.get_missing_keys(('Edge', 'NormalizedEnergy',
                                              'NormalizedSignal'))
    if missing_keys:
        _logger.error('missing keys:', missing_keys, 'unable to compute post plot')
        return None

    e0 = spectrum['Edge']
    x = spectrum['NormalizedEnergy'] - e0
    y = spectrum["NormalizedSignal"]
    return _CurveOperation(x=x, y=y, legend='Post', yaxis=None)


def _pre_plot(spectrum):
    assert isinstance(spectrum, Spectrum)
    missing_keys = spectrum.get_missing_keys(('Edge', 'NormalizedEnergy',
                                              'NormalizedBackground'))
    if missing_keys:
        _logger.error('missing keys:', missing_keys, 'unable to compute pre plot')
        return None

    e0 = spectrum['Edge']
    x = spectrum['NormalizedEnergy'] - e0
    y = spectrum["NormalizedBackground"]
    return _CurveOperation(x=x, y=y, legend='Pre', yaxis=None)


class NormalizationWindow(qt.QMainWindow):
    def __init__(self, parent=None):
        qt.QMainWindow.__init__(self, parent)

        # xas object viewer
        mapKeys = ['Mu', 'NormalizedMu', 'NormalizedSignal', 'NormalizedBackground']
        self.xasObjViewer = XasObjectViewer(mapKeys=mapKeys)
        self.xasObjViewer._spectrumView._plot.getXAxis().setLabel("Energy (eV)")
        self.xasObjViewer._spectrumView._plot.getYAxis().setLabel("Absorption (a.u.)")
        self.setCentralWidget(self.xasObjViewer)
        self._pymcaWindow = XASNormalizationParameters(parent=self)
        dockWidget = qt.QDockWidget(parent=self)

        # pymca window
        dockWidget.setWidget(self._pymcaWindow)
        self.addDockWidget(qt.Qt.RightDockWidgetArea, dockWidget)
        dockWidget.setAllowedAreas(qt.Qt.RightDockWidgetArea | qt.Qt.LeftDockWidgetArea)
        dockWidget.setFeatures(qt.QDockWidget.NoDockWidgetFeatures)

        # legend selector
        self.legendDockWidget = LegendSelector.LegendsDockWidget(parent=self,
                                                                 plot=self.xasObjViewer._spectrumView._plot)
        self.legendDockWidget.setAllowedAreas(qt.Qt.RightDockWidgetArea | qt.Qt.LeftDockWidgetArea)
        self.legendDockWidget.setFeatures(qt.QDockWidget.NoDockWidgetFeatures)
        self.addDockWidget(qt.Qt.RightDockWidgetArea, self.legendDockWidget)

        # volume key selection
        self.addDockWidget(qt.Qt.RightDockWidgetArea,
                           self.xasObjViewer._mapView.keySelectionDocker)

        # plot settings
        for ope in (_spectrum_plot, _normalized_spectrum_plot, _post_plot,
                    _pre_plot):
            self.xasObjViewer._spectrumView.addCurveOperation(ope)

        self.setWindowFlags(qt.Qt.Widget)

        # connect signal / slot
        self.xasObjViewer.viewTypeChanged.connect(self._updateLegendView)

        # set up
        self._updateLegendView()

    def getNCurves(self):
        return len(self.xasObjViewer._spectrumView._plot.getAllCurves())

    def _updateLegendView(self):
        self.legendDockWidget.setVisible(self.xasObjViewer.getViewType() is ViewType.spectrum)
        self.xasObjViewer._mapView.keySelectionDocker.setVisible(self.xasObjViewer.getViewType() is ViewType.map)


class NormalizationOW(OWWidget):
    """
    Widget used for signal extraction
    """
    name = "normalization"
    id = "orange.widgets.xas.normalization"
    description = "Progress spectra normalization"
    icon = "icons/normalization.png"
    priority = 1
    category = "esrfWidgets"
    keywords = ["spectroscopy", "normalization"]

    want_main_area = True
    resizing_enabled = True

    inputs = [("spectra", XASObject, "process")]

    outputs = [("spectra", XASObject), ("curves", tuple)]

    process_function = xas.core.process.normalization.PyMca_normalization

    _pymcaSettings = Setting(dict())
    """Store the configuration of the PyMca XASClass"""

    def __init__(self):
        OWWidget.__init__(self)
        self._latest_xas_obj = None
        self._window = NormalizationWindow(parent=self)
        layout = gui.vBox(self.mainArea, 'fourier transform').layout()
        layout.addWidget(self._window)
        self._window.xasObjViewer.setWindowTitle('spectra')

        # progress
        self._progress = gui.ProgressBar(self, 100)

        # manage settings
        if self._pymcaSettings != dict():
            self._window._pymcaWindow.setParameters(self._pymcaSettings)

        # connect signals / slots
        if hasattr(self._window._pymcaWindow, 'sigNormalizationParametersSignal'):
            _sig = self._window._pymcaWindow.sigNormalizationParametersSignal
        else:
            _sig = self._window._pymcaWindow.sigXASNormalizationParametersSignal
        _sig.connect(self._updateProcess)

    def process(self, xas_obj):
        if xas_obj is None:
            return

        # TODO: move process to multiprocessing + add advancement progress
        self._latest_xas_obj = xas_obj.copy()
        process_obj = QPyMca_normalization()
        process_obj._advancement.sigProgress.connect(self._setProgressValue)
        process_obj.setProperties({'_pymcaSettings': self._window._pymcaWindow.getParameters()})
        xas_obj = process_obj.process(xas_obj)
        process_obj._advancement.sigProgress.disconnect(self._setProgressValue)
        self._window.xasObjViewer.setXASObj(xas_obj=xas_obj)

        # emit signal for the plot
        self.send("spectra", xas_obj)

    def _updateProcess(self):
        self._update_settings()
        if self._latest_xas_obj:
            self.process(self._latest_xas_obj)

    def _update_settings(self):
        self._pymcaSettings = self._window._pymcaWindow.getParameters()

    def _setProgressValue(self, value):
        self._progress.widget.progressBarSet(value)


class QPyMca_normalization(xas.core.process.normalization.PyMca_normalization):
    def __init__(self):
        xas.core.process.normalization.PyMca_normalization.__init__(self)
        self._advancement = QProgress('normalization')
