# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016-2017 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/

__authors__ = ["H. Payno"]
__license__ = "MIT"
__date__ = "06/11/2019"

from xas.core.types import XASObject
from .progress import Progress
from PyMca5.PyMcaPhysics.xas.XASClass import XASClass
from PyMca5.PyMcaPhysics.xas.XASClass import e2k
import logging

_logger = logging.getLogger(__name__)


def _process_spectrum(spectrum, configuration):
    """process a single spectrum"""
    pymca_xas = XASClass()
    pymca_xas.setSpectrum(energy=spectrum.energy,
                          mu=spectrum.mu)
    pymca_xas.setConfiguration(configuration)
    spectrum.update(pymca_xas.processSpectrum())

    if 'Energy' not in spectrum or 'Mu' not in spectrum:
        _logger.error('Energy and or Mu is/are not specified, unable to '
                      'compute exafs')
        return None, None

    if 'EXAFSKValues' in spectrum and 'Mu' in spectrum:
        kValues = spectrum['EXAFSKValues']
    else:
        if 'Mu' not in spectrum:
            print('********************** computing **********************')
        if 'Edge' not in spectrum:
            spectrum.update(pymca_xas.processSpectrum())
        e0 = spectrum['Edge']
        energy = spectrum['Energy']
        kValues = e2k(energy - e0)
        spectrum['EXAFSKValues'] = kValues
    mu = spectrum.mu
    spectrum['EXAFS'] = pymca_xas.postEdge(k=kValues, mu=mu)

    return configuration, spectrum


def pymca_exafs(xas_obj):
    """

    :param xas_obj: object containing the configuration and spectra to process
    :type: Union[XASObject, dict]
    :return: spectra dict
    :rtype: XASObject
    """
    exafs_obj = PyMca_exafs()
    return exafs_obj.process(xas_obj=xas_obj)


class PyMca_exafs(object):
    """Process spectra for exafs and get information about the processing
    advancement"""
    def __init__(self):
        self._advancement = Progress('exafs')
        self._settings = None

    def setProperties(self, properties):
        if '_pymcaSettings' in properties:
            self._settings = properties['_pymcaSettings']

    def process(self, xas_obj):
        if isinstance(xas_obj, dict):
            _xas_obj = XASObject.from_dict(xas_obj)
        else:
            _xas_obj = xas_obj
        if self._settings:
            _xas_obj.configuration['EXAFS'] = self._settings

        self._advancement.startProcess()
        for i_spectrum, spectrum in enumerate(_xas_obj.spectra):
            _xas_obj.configuration, _xas_obj.spectra[i_spectrum] = _process_spectrum(spectrum=spectrum,
                                                                                     configuration=_xas_obj.configuration)
            self._advancement.setAdvancement(int(i_spectrum / _xas_obj.n_spectrum * 100))
        self._advancement.endProcess()
        assert isinstance(_xas_obj, XASObject)
        return _xas_obj

    __call__ = process
