# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016-2017 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/

__authors__ = ["H. Payno"]
__license__ = "MIT"
__date__ = "06/11/2019"

from xas.core.types import XASObject, Spectrum
from PyMca5.PyMcaPhysics.xas.XASClass import XASClass
from .progress import Progress
import logging

_logger = logging.getLogger(__name__)


def _process_spectrum(spectrum, configuration):
    assert spectrum is not None

    pymca_xas = XASClass()
    if spectrum.energy is None or spectrum.mu is None:
        raise ValueError('Energy and or Mu is/are not specified, unable to '
                         'compute exafs')
    pymca_xas.setSpectrum(energy=spectrum.energy,
                          mu=spectrum.mu)
    pymca_xas.setConfiguration(configuration)
    spectrum = Spectrum.from_dict(pymca_xas.processSpectrum())
    return configuration, spectrum


def pymca_k_weight(xas_obj):
    """

    :param xas_obj: object containing the configuration and spectra to process
    :type: Union[XASObject, dict]
    :return: spectra dict
    :rtype: dict
    """
    k_weight_obj = PyMca_k_weight()
    return k_weight_obj.process(xas_obj=xas_obj)


class PyMca_k_weight(object):
    def __init__(self):
        self._k_weight = None
        self._advancement = Progress('k weight')

    def setProperties(self, properties):
        if '_kWeightSetting' in properties:
            self._k_weight = properties['_kWeightSetting']

    def process(self, xas_obj):
        """
    
        :param xas_obj: object containing the configuration and spectra to process
        :type: Union[XASObject, dict]
        :return: spectra dict
        :rtype: dict
        """
        assert xas_obj is not None
        if isinstance(xas_obj, dict):
            _xas_obj = XASObject.from_dict(xas_obj)
        else:
            _xas_obj = xas_obj
        if self._k_weight:
            _xas_obj.configuration['SET_KWEIGHT'] = self._k_weight

        if 'SET_KWEIGHT' not in _xas_obj.configuration:
            _logger.warning(
                    'Missing configuration to know which value we should set '
                    'to k weight, will be set to 0 by default')
            _xas_obj.configuration['SET_KWEIGHT'] = 0

        for key in ('FT', 'EXAFS', 'Normalization'):
            if key not in _xas_obj.configuration:
                _xas_obj.configuration[key] = {}

        _xas_obj.configuration['KWeight'] = _xas_obj.configuration[
            'SET_KWEIGHT']
        _xas_obj.configuration['FT']['KWeight'] = _xas_obj.configuration[
            'SET_KWEIGHT']
        _xas_obj.configuration['EXAFS']['KWeight'] = _xas_obj.configuration[
            'SET_KWEIGHT']
        _xas_obj.configuration['Normalization']['KWeight'] = \
        _xas_obj.configuration[
            'SET_KWEIGHT']

        self._advancement.startProcess()
        for i_spectrum, spectrum in enumerate(_xas_obj.spectra):
            _xas_obj.configuration, _xas_obj.spectra[
                i_spectrum] = _process_spectrum(spectrum=spectrum,
                                                configuration=_xas_obj.configuration)
            self._advancement.setAdvancement(int((i_spectrum * _xas_obj.n_spectrum) * 100))

        self._advancement.endProcess()
        return _xas_obj

    __call__ = process
