# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016-2017 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/
"""Tools to visualize spectra"""


__authors__ = ["H. Payno"]
__license__ = "MIT"
__date__ = "04/07/2019"


from xas.core.types import XASObject
from xas.io import read_pymca_xas
from silx.gui import qt
from silx.gui.plot import Plot1D
from silx.gui.plot.StackView import StackViewMainWindow
from silx.utils.enum import Enum
from silx.gui.widgets.FrameBrowser import HorizontalSliderWithBrowser
from silx.gui import icons as silx_icons
from silx.gui.colors import Colormap
from collections import namedtuple


class ViewType(Enum):
    map = 0,
    spectrum = 1,


class _SpectrumViewAction(qt.QAction):
    def __init__(self, parent=None):
        qt.QAction.__init__(self, 'spectrum view', parent=parent)
        spectrum_icon = silx_icons.getQIcon("item-1dim")
        self.setIcon(spectrum_icon)
        self.setCheckable(True)


class _MapViewAction(qt.QAction):
    def __init__(self, parent=None):
        qt.QAction.__init__(self, 'map view', parent=parent)
        map_icon = silx_icons.getQIcon("image")
        self.setIcon(map_icon)
        self.setCheckable(True)


class XasObjectViewer(qt.QMainWindow):
    """Viewer dedicated to view a XAS object"""

    viewTypeChanged = qt.Signal()
    """emitted when the view type change"""

    def __init__(self, parent=None, mapKeys=None):
        qt.QMainWindow.__init__(self, parent)
        self.setWindowFlags(qt.Qt.Widget)

        # main stack widget
        self._mainWidget = qt.QWidget(parent=self)
        self._mainWidget.setLayout(qt.QVBoxLayout())
        self.setCentralWidget(self._mainWidget)
        # map view
        self._mapView = MapViewer(parent=self, keys=mapKeys)
        self._mainWidget.layout().addWidget(self._mapView)

        # spectrum view
        self._spectrumView = SpectrumViewer(parent=self)
        self._mainWidget.layout().addWidget(self._spectrumView)

        # add toolbar
        toolbar = qt.QToolBar('')
        toolbar.setIconSize(qt.QSize(32, 32))
        self.view_actions = qt.QActionGroup(self)
        self._spectrumViewAction = _SpectrumViewAction()
        self.view_actions.addAction(self._spectrumViewAction)
        self._mapViewAction = _MapViewAction()
        self.view_actions.addAction(self._mapViewAction)

        toolbar.addAction(self._spectrumViewAction)
        toolbar.addAction(self._mapViewAction)

        self.addToolBar(qt.Qt.LeftToolBarArea, toolbar)
        toolbar.setMovable(False)

        # connect signal / Slot
        self._mapViewAction.triggered.connect(self._updateView)
        self._spectrumViewAction.triggered.connect(self._updateView)

        # initialize
        self._spectrumViewAction.setChecked(True)
        self._updateView()

    def _updateView(self, *arg, **kwargs):
        self._mapView.setVisible(self.getViewType() is ViewType.map)
        self._spectrumView.setVisible(self.getViewType() is ViewType.spectrum)
        self.viewTypeChanged.emit()

    def getViewType(self):
        if self._mapViewAction.isChecked():
            return ViewType.map
        elif self._spectrumViewAction.isChecked():
            return ViewType.spectrum

    def setXASObj(self, xas_obj):
        self._mapView.clear()
        self._spectrumView.clear()

        self._mapView.setXasObject(xas_obj)
        self._spectrumView.setXasObject(xas_obj)


class MapViewer(qt.QWidget):
    def __init__(self, parent=None, keys=None):
        """
        
        :param parent: 
        :param keys: volume keys to display for the xasObject (Mu,
        NormalizedMu...)
        """
        assert keys is not None
        self._xasObj = None
        qt.QWidget.__init__(self, parent=parent)
        self.setLayout(qt.QVBoxLayout())
        self._mainWindow = StackViewMainWindow(parent=parent)
        self.layout().addWidget(self._mainWindow)
        self.layout().setContentsMargins(0, 0, 0, 0)
        self.layout().setSpacing(0.0)

        self._mainWindow.setKeepDataAspectRatio(True)
        self._mainWindow.setColormap(Colormap(name='temperature'))

        # define the keys combobox
        self._keyWidget = qt.QWidget(parent=self)
        self._keyWidget.setLayout(qt.QHBoxLayout())
        self._keyComboBox = qt.QComboBox(parent=self._keyWidget)
        for key in keys:
            self._keyComboBox.addItem(key)
        self._keyWidget.layout().addWidget(qt.QLabel('view: '))
        self._keyWidget.layout().addWidget(self._keyComboBox)
        self.keySelectionDocker = qt.QDockWidget(parent=self)
        self.keySelectionDocker.setContentsMargins(0, 0, 0, 0)
        self._keyWidget.layout().setContentsMargins(0, 0, 0, 0)
        self._keyWidget.layout().setSpacing(0.0)
        self.keySelectionDocker.setWidget(self._keyWidget)
        # self._mainWindow.addDockWidget(qt.Qt.TopDockWidgetArea, dockWidget)
        self.keySelectionDocker.setAllowedAreas(qt.Qt.TopDockWidgetArea)
        self.keySelectionDocker.setFeatures(qt.QDockWidget.NoDockWidgetFeatures)

        # connect signal / slot
        self._keyComboBox.currentTextChanged.connect(self._updateView)

    def clear(self):
        self._mainWindow.clear()

    def getActiveKey(self):
        return self._keyComboBox.currentText()

    def setXasObject(self, xas_obj):
        self._xasObj = xas_obj
        self._updateView()

    def _updateView(self, *args, **kwargs):
        if self._xasObj is None:
            return
        # set the map view
        spectra_volume = XASObject._spectra_volume(spectra=self._xasObj.spectra,
                                                   dim_1=self._xasObj.dim1,
                                                   dim_2=self._xasObj.dim2,
                                                   key=self.getActiveKey())
        self._mainWindow.setStack(spectra_volume)



class _ExtendedSliderWithBrowser(HorizontalSliderWithBrowser):
    def __init__(self, parent=None, name=None):
        HorizontalSliderWithBrowser.__init__(self, parent)
        self.layout().insertWidget(0, qt.QLabel(str(name + ':')))


class _CurveOperation(object):
    def __init__(self, x, y, legend, yaxis=None, linestyle=None, symbol=None,
                 color=None, ylabel=None):
        self.x = x
        self.y = y
        self.legend = legend
        self.yaxis = yaxis
        self.linestyle = linestyle
        self.symbol = symbol
        self.color = color
        self.ylabel = ylabel


class SpectrumViewer(qt.QMainWindow):
    def __init__(self, parent=None):
        self._curveOperations = []
        """List of callaback to produce plot regarding the XASObject.
        Callback function should return a _curve_operation"""
        qt.QMainWindow.__init__(self, parent)
        self.xas_obj = None
        self._plot = Plot1D(parent=self)
        self.setCentralWidget(self._plot)

        # frame browsers
        dockWidget = qt.QDockWidget(parent=self)

        frameBrowsers = qt.QWidget(parent=self)
        frameBrowsers.setLayout(qt.QVBoxLayout())
        frameBrowsers.layout().setContentsMargins(0, 0, 0, 0)

        self._dim1FrameBrowser = _ExtendedSliderWithBrowser(parent=self,
                                                            name='dim 1')
        frameBrowsers.layout().addWidget(self._dim1FrameBrowser)
        self._dim2FrameBrowser = _ExtendedSliderWithBrowser(parent=self,
                                                            name='dim 2')
        frameBrowsers.layout().addWidget(self._dim2FrameBrowser)
        dockWidget.setWidget(frameBrowsers)

        self.addDockWidget(qt.Qt.BottomDockWidgetArea, dockWidget)
        dockWidget.setAllowedAreas(qt.Qt.BottomDockWidgetArea)
        dockWidget.setFeatures(qt.QDockWidget.NoDockWidgetFeatures)

        # connect signal / slot
        self._dim1FrameBrowser.valueChanged.connect(self._updateSpectrumDisplayed)
        self._dim2FrameBrowser.valueChanged.connect(self._updateSpectrumDisplayed)

    def addCurveOperation(self, callback):
        """register an curve to display from Xasobject keys, and a legend"""
        self._curveOperations.append(callback)

    def clearCurveOperations(self):
        """Remove all defined curve operation"""
        self._curveOperations.clear()

    def setXasObject(self, xas_obj):
        self.xas_obj = xas_obj
        assert self.xas_obj.dim1 >= 0
        assert self.xas_obj.dim2 >= 0
        self._dim1FrameBrowser.setRange(0, self.xas_obj.dim1-1)
        self._dim2FrameBrowser.setRange(0, self.xas_obj.dim2-1)

    def clear(self):
        self._plot.clear()

    def _updateSpectrumDisplayed(self, *args, **kwargs):
        if self.xas_obj is None:
            return
        dim1_index = self._dim1FrameBrowser.value()
        dim2_index = self._dim2FrameBrowser.value()
        if dim1_index < 0 or dim2_index < 0:
            return

        assert dim1_index >= 0
        assert dim2_index >= 0

        spectrum = self.xas_obj.getSpectrum(dim1_index, dim2_index)
        for operation in self._curveOperations:
            res = operation(spectrum)
            if res is not None:
                assert isinstance(res, _CurveOperation)
                self._plot.addCurve(x=res.x,
                                    y=res.y,
                                    legend=res.legend,
                                    yaxis=res.yaxis,
                                    linestyle=res.linestyle,
                                    symbol=res.symbol,
                                    color=res.color,
                                    ylabel=res.ylabel,
                                    )

    def clear(self):
        self._plot.clear()
        self._dim1FrameBrowser.setMaximum(-1)
        self._dim2FrameBrowser.setMaximum(-1)


if __name__ == '__main__':
    from silx.io.url import DataUrl

    file_name = "S5_nuit1_merge_new_crop.h5"
    url_spectra = DataUrl(file_path=file_name,
                          data_path='/data/NXdata/data',
                          scheme='silx')
    url_channel = DataUrl(file_path=file_name,
                          data_path='/data/NXdata/Channel',
                          scheme='silx')

    xas_obj = read_pymca_xas(spectra_url=url_spectra, channel_url=url_channel)

    # import numpy
    # from xas.core.utils import spectra as spectra_utils
    # spectra = spectra_utils.create_dataset(shape=(256, 32, 64))
    # energy = numpy.linspace(start=3.25, stop=3.69, num=256)
    # xas_obj = XASObject(spectra=spectra, energy=energy, configuration=None)

    app = qt.QApplication([])
    viewer = XasObjectViewer()
    viewer.setXASObj(xas_obj=xas_obj)
    viewer.show()
    app.exec_()
