# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2016-2017 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/

__authors__ = ["H. Payno"]
__license__ = "MIT"
__date__ = "06/12/2019"

import logging
import h5py
from silx.io import utils
from silx.io.url import DataUrl
from xas.core.utils.pymca import read_spectrum
from xas.core.types import XASObject
from silx.utils.enum import Enum

_logger = logging.getLogger(__name__)


DEFAULT_SPECTRA_PATH = '/data/NXdata/data'

DEFAULT_CHANNEL_PATH = '/data/NXdata/Channel'

DEFAULT_CONF_PATH = '/configuration'


class InputType(Enum):
    single_spectrum = '*.dat'
    multiple_spectra = '*.h5'


def read_pymca_xas_from_file(file_path):
    """
    
    :param str file_path: 
    :return: `.XASObject`
    """
    if file_path.endswith('.dat'):
        return read_pymca_xas(
                spectra_url=DataUrl(file_path=file_path,
                                    scheme='PyMca'),
                channel_url=DataUrl(file_path=file_path,
                                    scheme='PyMca'),
        )
    elif h5py.is_hdf5(file_path):
        return read_pymca_xas(
                spectra_url=DataUrl(file_path=file_path,
                                    scheme='silx',
                                    data_path=DEFAULT_SPECTRA_PATH),
                channel_url=DataUrl(file_path=file_path,
                                    scheme='silx',
                                    data_path=DEFAULT_CHANNEL_PATH),
                config_url=DataUrl(file_path=file_path,
                                   scheme='silx',
                                   data_path='configuration'))
    else:
        raise ValueError('file type not managed, unable to load')


def read_pymca_xas(spectra_url, channel_url, config_url=None):
    """
    Read the given spectra url and the config url if any
    
    :param Union[DataUrl, str] spectra_url: 
    :param DataUrl config_url: 
    :return: PyMcaSpectrum
    """
    def get_url(original_url, name):
        url_ = original_url
        if type(url_) is str:
            try:
                url_ = DataUrl(path=url_)
            except:
                url_ = DataUrl(file_path=url_, scheme='PyMca')

        if not isinstance(url_, DataUrl):
            raise TypeError('given input for, ', name, 'is invalid')
        return url_

    _spectra_url = get_url(original_url=spectra_url, name='spectra')
    _energy_url = get_url(original_url=channel_url, name='energy')
    _config_url = config_url
    if _config_url == '':
        _config_url = None
    if not (_config_url is None or isinstance(_config_url, DataUrl)):
        raise TypeError('given input for configuration is invalid')

    def load_data(data_url, name):
        if data_url is None:
            return None
        if data_url.scheme() in ('PyMca', 'PyMca5'):
            assert name in ('spectra', 'energy')
            energy, mu = read_spectrum(data_url.file_path())
            if name == 'spectra':
                return mu.reshape(mu.shape[0], 1, 1)
            else:
                return energy
        else:
            if data_url.is_valid():
                return utils.get_data(data_url)
            else:
                _logger.warning('invalid url for', name, ',  will not load it')

    spectra = load_data(_spectra_url, name='spectra')
    energy = load_data(_energy_url, name='energy')
    configuration = load_data(_config_url, name='configuration')

    return XASObject(spectra=spectra, energy=energy, configuration=configuration)


class Writer_pymca_cas(object):
    """
    class to write the output file. In this case we need a class in order to
    setup the output file before
    """
    def __init__(self):
        self._output_file = None

    def setProperties(self, properties):
        if '_output_file_setting' in properties:
            self._output_file = properties['_output_file_setting']

    def dump_pymca_xas(self, xas_obj):
        if not self._output_file:
            _logger.warning('no output file defined, please give path to the output file')
            self._output_file = input()
        if isinstance(xas_obj, dict):
            _xas_obj = XASObject.from_dict(xas_obj)
        else:
            _xas_obj = xas_obj
        _logger.info(('dump xas obj to', self._output_file))
        _xas_obj.dump(self._output_file)

    __call__ = dump_pymca_xas
