# coding: utf-8
# /*##########################################################################
#
# Copyright (c) 2017-2019 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/
__authors__ = ["H. Payno"]
__license__ = "MIT"
__date__ = "06/12/2019"


import unittest
from PyMca5.PyMcaDataDir import PYMCA_DATA_DIR
from xas.pushworkflow.scheme.node import Node
from xas.pushworkflow.scheme.link import Link
from xas.pushworkflow.scheme.scheme import Scheme
from xas.app.process import exec_ as exec_workflow
import xas.core.process.normalization
import xas.core.process.k_weight
import xas.core.process.exafs
import xas.core.process.ft
from xas.core.types import XASObject
import xas.io
import os


class TestWorkflowFunctions(unittest.TestCase):
    """Test some processing workflow"""
    def setUp(self):
        unittest.TestCase.setUp(self)

        read_task = Node(callback=xas.io.read_pymca_xas_from_file)
        normalization_task = Node(
            callback=xas.core.process.normalization.pymca_normalization)
        k_weight_task = Node(callback=xas.core.process.k_weight.pymca_k_weight)
        exafs_task = Node(callback=xas.core.process.exafs.pymca_exafs)
        ft_task = Node(callback=xas.core.process.ft.pymca_ft)

        nodes = (
        read_task, normalization_task, k_weight_task, exafs_task, ft_task)

        links = [
            Link(source_node=read_task, source_channel='spectra',
                 sink_node=normalization_task, sink_channel='spectra'),
            Link(source_node=normalization_task, source_channel='spectra',
                 sink_node=k_weight_task, sink_channel='spectra'),
            Link(source_node=k_weight_task, source_channel='spectra',
                 sink_node=exafs_task, sink_channel='spectra'),
            Link(source_node=exafs_task, source_channel='spectra',
                 sink_node=ft_task, sink_channel='spectra'),
        ]

        self.scheme = Scheme(nodes=nodes, links=links)

        self.data_file = os.path.join(PYMCA_DATA_DIR, "EXAFS_Cu.dat")

    def testProcess(self):
        out = exec_workflow(scheme=self.scheme, input_=self.data_file)
        assert isinstance(out, dict)
        xas_obj_out = XASObject.from_dict(out)
        assert 'FT' in xas_obj_out.spectra[0]
        assert 'FTRadius' in xas_obj_out.spectra[0].ft


class TestWorkflowCallableClasses(unittest.TestCase):
    """Test some processing workflow"""
    def setUp(self):
        unittest.TestCase.setUp(self)

        read_task = Node(callback=xas.io.read_pymca_xas_from_file)
        normalization_task = Node(
            callback=xas.core.process.normalization.PyMca_normalization)
        k_weight_task = Node(callback=xas.core.process.k_weight.PyMca_k_weight)
        exafs_task = Node(callback=xas.core.process.exafs.PyMca_exafs)
        ft_task = Node(callback=xas.core.process.ft.PyMca_ft)

        nodes = (
        read_task, normalization_task, k_weight_task, exafs_task, ft_task)

        links = [
            Link(source_node=read_task, source_channel='spectra',
                 sink_node=normalization_task, sink_channel='spectra'),
            Link(source_node=normalization_task, source_channel='spectra',
                 sink_node=k_weight_task, sink_channel='spectra'),
            Link(source_node=k_weight_task, source_channel='spectra',
                 sink_node=exafs_task, sink_channel='spectra'),
            Link(source_node=exafs_task, source_channel='spectra',
                 sink_node=ft_task, sink_channel='spectra'),
        ]

        self.scheme = Scheme(nodes=nodes, links=links)

        self.data_file = os.path.join(PYMCA_DATA_DIR, "EXAFS_Cu.dat")

    def testProcess(self):
        out = exec_workflow(scheme=self.scheme, input_=self.data_file)
        assert isinstance(out, dict)
        xas_obj_out = XASObject.from_dict(out)
        assert 'FT' in xas_obj_out.spectra[0]
        assert 'FTRadius' in xas_obj_out.spectra[0].ft
        assert 'FT' in xas_obj_out.configuration
        assert 'Normalization' in xas_obj_out.configuration


def suite():
    test_suite = unittest.TestSuite()
    for ui in (TestWorkflowFunctions, TestWorkflowCallableClasses):
        test_suite.addTest(unittest.defaultTestLoader.loadTestsFromTestCase(ui))
    return test_suite


if __name__ == '__main__':
    unittest.main(defaultTest="suite")
